#include "../lib/sqlite3.hpp"

namespace marcelb::SQlite3 {

SQLite3::SQLite3(const string _path, bool _keepOpen) {
   path = _path;
   keepOpen = _keepOpen;

   if (keepOpen) {
      if (open()) {
         throw string("[ERROR] Unable to open database ");
      }
   }
}

bool SQLite3::open() {
   uint operationStatus = sqlite3_open(path.c_str(), &db);
   return operationStatus != SQLITE_OK;
}

bool SQLite3::close() {
   uint operationStatus = sqlite3_close(db);
   db = NULL;
   return operationStatus != SQLITE_OK;
}
/*
map<string, vector<string>> SQLite3::query(const string sql_command) {
   io.lock();
   if (!keepOpen) {
      if (open()) {
         io.unlock();
         throw string("[ERROR] Unable to open database ");
      }
   }

   bool r = true;
   char *messaggeError;
   uint exec_status;
   exec_status = sqlite3_exec(db, sql_command.c_str(), callback, NULL, &messaggeError);

   string result;
   if (exec_status != SQLITE_OK) {
      sqlite3_free(messaggeError);
      r = false;
   }
   else  {
      result = responseDatabase;
      responseDatabase.clear();
   }

   if (!keepOpen) {
      if(close()) {
         io.unlock();
         throw string("[ERROR] Unable to close database ");
      }
   }

   map<string, vector<string>> _parsed;

   if (!r) {
      io.unlock();
      throw string("[ERROR] Unable to read database ");
   }

   else {
      _parsed = parse(result);
   }
   io.unlock();
   return _parsed;
}
*/

map<string, vector<string>> SQLite3::query(const string& sql_command, const vector<string>& values) {
    lock_guard<mutex> lock(io);
    map<string, vector<string>> results;
    
    if (!keepOpen) {
        if (open()) {
            throw string("[ERROR] Unable to open database ");
        }
    }

    sqlite3_stmt* stmt;
    if (sqlite3_prepare_v2(db, sql_command.c_str(), -1, &stmt, nullptr) != SQLITE_OK) {
        throw string("[ERROR] Failed to prepare statement: ") + sqlite3_errmsg(db);
    }

    // Binduj parametre
    for (size_t i = 0; i < values.size(); ++i) {
        sqlite3_bind_text(stmt, static_cast<int>(i + 1), values[i].c_str(), -1, SQLITE_TRANSIENT);
    }

    // Dohvati rezultate
    int columnCount = sqlite3_column_count(stmt);
    while (sqlite3_step(stmt) == SQLITE_ROW) {
        for (int i = 0; i < columnCount; ++i) {
            const char* columnName = sqlite3_column_name(stmt, i);
            const char* columnValue = (const char*)sqlite3_column_text(stmt, i);
            results[columnName].push_back(columnValue ? columnValue : "NULL");
        }
    }

    sqlite3_finalize(stmt); // Oslobodi resurse vezane uz izjavu

    if (!keepOpen) {
        if (close()) {
            throw string("[ERROR] Unable to close database ");
        }
    }

    return results;
}

map<string, vector<string>> SQLite3::parse(const string& result) {
   string a = result;
   map<string, vector<string>> parsed;

   while (a.find("\n") < a.length()) {
      uint lineend = a.find("\n");
      string oneline = a.substr(0, lineend);
      a.erase(0, lineend+1);
      
      size_t sep = oneline.find(" = ");
      string key = oneline.substr(0, sep);
      string value = oneline.substr(sep + 3, oneline.length());
      value = value.substr(0, value.length());
      parsed[key].push_back(value);
   }

   return parsed;
}

SQLite3::~SQLite3() {
   if(close()) {
      // throw string("[ERROR] Unable to close database ");
   }
}


/**
 * Callback funkcija za SQLite3 
*/

static int callback(void* data, int argc, char** argv, char** azColName) {
   int i;
   char res[1000];
   for (i = 0; i < argc; i++) {
      sprintf(res, "%s = %s", azColName[i], argv[i] ? argv[i] : "NULL");
      responseDatabase += string(res) + string("\n");
   }
   return 0;
}


}